import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {
  MatTableDataSource
} from "@angular/material/table";
import {LeafGamesystemCalculator} from "../../product-gamesystem-editor/LeafGamesystemCalculator";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {ProductGamesystem} from "../../../../project/game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../project/game-model/gamesystems/SimpleGamesystem";
import {ProductState} from "../../../../project/game-model/gamesystems/states/ProductState";
import {State} from "../../../../project/game-model/gamesystems/states/State";
import {TemplateElement} from "../../../../project/game-model/templates/TemplateElement";
import {ProductTemplateSystem} from "../../../../project/game-model/templates/productGamesystem/ProductTemplateSystem";

@Component({
  selector: 'app-product-state-editor',
  templateUrl: './product-state-editor.component.html',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
  styleUrl: './product-state-editor.component.scss'
})
export class ProductStateEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  @Input() templateElement: TemplateElement | undefined
  @Output('onOpenGamesystemEditor') openGamesystemEditorEmitter = new EventEmitter<SimpleGamesystem>();
  displayedColumns: string[] = [];
  internalColumnNames: string[] = []

  expandedColumns: string[] = []
  datasource = new MatTableDataSource<ProductState>();

  expandedElement: ProductState | null = null;

  ngOnInit() {
    this.gamesystem!.generateFromChildsystems();
    this.generateColumnNamesRecursively(this.gamesystem!, "");
    this.displayedColumns.push('Initial');

    this.internalColumnNames = this.displayedColumns.concat();

    this.renameDuplicateColumnNames(this.internalColumnNames)

    this.expandedColumns = [...this.internalColumnNames, 'expand'];

    if(this.templateElement == undefined) {
      this.datasource.data = this.gamesystem!.states;
    } else if(this.gamesystem instanceof ProductTemplateSystem) {
      console.log("Product Template System: ", this.gamesystem!.stateMap.get(this.templateElement))
      this.datasource.data = this.gamesystem!.stateMap.get(this.templateElement)!
    }

    this.datasource.filterPredicate = (data: ProductState, filter: string) => {
      const leaf_states = LeafGamesystemCalculator.calcLeafStates(data);
      return leaf_states.some((state) => state.stateLabel.toLowerCase().includes(filter))
    }
  }

  private renameDuplicateColumnNames(columnNames: string[]) {
    for(let i=0; i<columnNames.length; i++) {
      for(let j=i; j<columnNames.length; j++) {
        if(i === j) {
          continue;
        }

        if(columnNames[i] === columnNames[j]) {
          columnNames[j] = columnNames[j] + "-" + i
        }
      }
    }
  }

  generateColumnNamesRecursively(gamesystem: ProductGamesystem, nestedColumnName: string) {
    gamesystem.innerGamesystems.forEach(innerGamesystem => {
      if(innerGamesystem instanceof SimpleGamesystem) {
        this.displayedColumns.push(nestedColumnName + innerGamesystem.componentName);
      } else {
        this.generateColumnNamesRecursively(innerGamesystem as ProductGamesystem, nestedColumnName + innerGamesystem.componentName + ".");
      }
    })
  }


  getLeafState(state: State<any>, i: number) {
    return LeafGamesystemCalculator.calcLeafStates(state)[i];
  }

  clickOnInnerState(leafIndex: number) {
    const leaf_gamesystems = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem!);
    const clicked_gamesystem = leaf_gamesystems[leafIndex];
    this.openGamesystemEditorEmitter.emit(clicked_gamesystem);
  }

  applyFilter(event: KeyboardEvent) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.datasource.filter = filterValue.trim().toLowerCase();
  }

}
