import {ProductGamesystem} from "../../gamesystems/ProductGamesystem";
import {TemplateGamesystem} from "../TemplateGamesystem";
import {TemplateElement} from "../TemplateElement";
import {ProductState} from "../../gamesystems/states/ProductState";
import {ProductTransition} from "../../gamesystems/transitions/ProductTransition";
import {SimpleGamesystem} from "../../gamesystems/SimpleGamesystem";
import {GameModel} from "../../GameModel";
import {TemplateType} from "../TemplateType";
import {TemplateProductSystemGenerator} from "../../gamesystems/productSystemGenerator/TemplateProductSystemGenerator";
import {
  SymmetricProductTemplateGenerator
} from "../../gamesystems/productSystemGenerator/SymmetricProductTemplateGenerator";
import {IsolatedTemplateStateGenerator} from "../../gamesystems/productSystemGenerator/IsolatedTemplateStateGenerator";
import {
  IsolatedSymmetricTemplateStateGenerator
} from "../../gamesystems/productSystemGenerator/IsolatedSymmetricTemplateStateGenerator";

export class ProductTemplateSystem extends ProductGamesystem implements TemplateGamesystem{

  stateMap: Map<TemplateElement, ProductState[]> = new Map();
  transitionMap: Map<TemplateElement, ProductTransition[]> = new Map<TemplateElement, ProductTransition[]>()
  templateType: TemplateType
  symmetric: boolean = false


  constructor(gamesystemName: string, gamesystemDescription: string, templateType: TemplateType) {
    super(gamesystemName, gamesystemDescription);
    this.templateType = templateType;
  }

  addTemplateElement(templateElement: TemplateElement): void {
    if(this.symmetric) {
      const symmetricGenerator = new SymmetricProductTemplateGenerator(this, templateElement);
      symmetricGenerator.generateFromChildsystems()

      if(this.generateIsolatedStates) {
        const isolatedTemplateStateGenerator = new IsolatedSymmetricTemplateStateGenerator(this, templateElement);
        isolatedTemplateStateGenerator.generateIsolatedProductStates();
        console.log(this.states)
        console.log("Generate symmetric isolated states")
      } else {
        console.log("Do not generate symmetric isolated states")
      }
    } else {
      const productTemplateGenerator = new TemplateProductSystemGenerator(this, templateElement);
      productTemplateGenerator.generateFromChildsystems()

      if(this.generateIsolatedStates) {
        const isolatedTemplateStateGenerator = new IsolatedTemplateStateGenerator(this, templateElement);
        isolatedTemplateStateGenerator.generateIsolatedProductStates();
      }
    }


  }

}
