import {SimpleState} from "../../game-model/gamesystems/states/SimpleState";
import {ScriptAccount} from "../../game-model/scriptAccounts/ScriptAccount";
import {ScriptAccountCondition} from "../../game-model/gamesystems/conditions/ScriptAccountCondition";
import {ScriptAccountConditionParser} from "./ScriptAccountConditionParser";
import {TemplateType} from "../../game-model/templates/TemplateType";
import {SimpleTemplateState} from "../../game-model/templates/simpleGamesystem/SimpleTemplateState";

export class StateParser {

    private conditionParser


    constructor(scriptAccounts: ScriptAccount[]) {
      this.conditionParser = new ScriptAccountConditionParser(scriptAccounts)
    }

    public  parseStates(stateData: any, templateType: TemplateType | undefined): SimpleState[] {
      const parsedStates: SimpleState[] = []
      for(let i=0; i<stateData.length; i++) {
        parsedStates.push(this.parseState(stateData[i], templateType))
      }

      return parsedStates;
    }

   private parseState(stateData: any, templateType: TemplateType | undefined): SimpleState {
      const initial = stateData.initial
      const stateLabel = stateData.stateLabel
      const stateDescription = stateData.stateDescription

      const conditions = this.conditionParser.parseStoredConditions(stateData.conditions)

      let simpleState;
      if(templateType == undefined) {
        simpleState = new SimpleState(stateLabel, stateDescription)
      } else {
        simpleState = new SimpleTemplateState(stateLabel, stateDescription)
      }
      simpleState.initial = initial;
      simpleState.conditions = conditions;

      return simpleState;
   }


}
